/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2003 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.dynamo.index;

import java.io.*;
import java.lang.ref.*;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.document.*;
import org.apache.lucene.index.*;
import org.apache.lucene.store.Directory;
import com.silverwrist.dynamo.except.*;
import com.silverwrist.dynamo.iface.*;
import com.silverwrist.dynamo.util.*;

class IndexServiceImpl implements IndexService
{
  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private QualifiedNameKey m_identity;
  private IndexManagerObject m_base;
  private Analyzer m_analyzer;
  private IndexDirectoryImpl m_directory;
  private DirectoryAutoCleanup m_cleanup;

  /*--------------------------------------------------------------------------------
   * Constructor
   *--------------------------------------------------------------------------------
   */

  IndexServiceImpl(IndexManagerObject base, QualifiedNameKey identity, int ndx, IndexOps ops, ReferenceQueue rq)
      throws DatabaseException, IndexException
  {
    String ana_klass = null;
    try
    { // Create the analyzer class.
      ana_klass = ops.getAnalyzerClassName(ndx);
      m_analyzer = (Analyzer)(Class.forName(ana_klass).newInstance());

    } // end try
    catch (ClassNotFoundException e)
    { // unable to find the Analyzer class
      IndexException ie = new IndexException(IndexServiceImpl.class,"IndexMessages","analyzer.class.notfound",e);
      ie.setParameter(0,ana_klass);
      throw ie;

    } // end catch
    catch (IllegalAccessException e)
    { // problem creating the analyzer class
      IndexException ie = new IndexException(IndexServiceImpl.class,"IndexMessages","analyzer.noCreate",e);
      ie.setParameter(0,ana_klass);
      throw ie;

    } // end catch
    catch (InstantiationException e)
    { // unable to instantiate the class
      IndexException ie = new IndexException(IndexServiceImpl.class,"IndexMessages","analyzer.noCreate",e);
      ie.setParameter(0,ana_klass);
      throw ie;

    } // end catch
    catch (ClassCastException e)
    { // bad class specified
      IndexException ie = new IndexException(IndexServiceImpl.class,"IndexMessages","analyzer.badType",e);
      ie.setParameter(0,ana_klass);
      throw ie;

    } // end catch

    m_base = base;
    m_identity = identity;
    m_directory = new IndexDirectoryImpl(ndx,ops);
    m_cleanup = new DirectoryAutoCleanup(this,m_directory,rq);

  } // end constructor

  /*--------------------------------------------------------------------------------
   * Internal operations
   *--------------------------------------------------------------------------------
   */

  private final String createTag(String namespace, String name, Object obj) throws IndexException
  {
    String objtag = m_base.getResolver(namespace,name).getResolverTag(obj);
    return namespace + "|" + name + "|" + objtag;

  } // end createTag

  /*--------------------------------------------------------------------------------
   * Implementations from interface IndexService
   *--------------------------------------------------------------------------------
   */

  public void addItem(String item_namespace, String item_name, Object item, String scope, java.util.Date date,
		      DynamoUser owner, String text) throws IndexException
  {
    // Create a new Lucene Document containing the item information.
    Document doc = new Document();
    doc.add(Field.Keyword("id",createTag(item_namespace,item_name,item)));
    doc.add(Field.Keyword("date",date));
    doc.add(Field.Keyword("owner",owner.getName()));
    doc.add(Field.Keyword("scope",scope));
    doc.add(Field.UnStored("text",text));

    try
    { // Use an IndexWriter to write it to the index.
      IndexWriter iwr = new IndexWriter(m_directory,m_analyzer,false);
      iwr.addDocument(doc);
      iwr.close();

    } // end try
    catch (IOException e)
    { // translate Lucene's IOException here
      IndexException ie = new IndexException(IndexServiceImpl.class,"IndexMessages","addItem.fail",e);
      ie.setParameter(0,item_namespace);
      ie.setParameter(1,item_name);
      ie.setParameter(2,m_identity.toString());
      throw ie;

    } // end catch

  } // end addItem

  public boolean deleteItem(String item_namespace, String item_name, Object item) throws IndexException
  {
    // Create a Lucene Term matching the given document.
    Term term = new Term("id",createTag(item_namespace,item_name,item));

    try
    { // Use an IndexReader to delete the appropriate document.
      IndexReader irdr = IndexReader.open(m_directory);
      int ndel = irdr.delete(term);
      irdr.close();
      return (ndel>0);

    } // end try
    catch (IOException e)
    { // translate Lucene's IOException here
      IndexException ie = new IndexException(IndexServiceImpl.class,"IndexMessages","deleteItem.fail",e);
      ie.setParameter(0,item_namespace);
      ie.setParameter(1,item_name);
      ie.setParameter(2,m_identity.toString());
      throw ie;

    } // end catch

  } // end deleteItem

} // end class IndexServiceImpl
