/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2002-03 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.dynamo.util;

import java.io.*;
import com.silverwrist.dynamo.HttpStatusCode;
import com.silverwrist.dynamo.except.RenderingException;
import com.silverwrist.dynamo.iface.*;

/**
 * A Dynamo output object that, when rendered, causes the browser to be sent an error message.  This object
 * may send any defined {@link com.silverwrist.dynamo.HttpStatusCode HttpStatusCode}.
 *
 * @author Eric J. Bowersox &lt;erbo@silcom.com&gt;
 * @version X
 */
public class HttpErrorMessage implements SelfRenderable
{
  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private HttpStatusCode m_code;    // status code to be sent
  private String m_message;         // error message to be sent

  /*--------------------------------------------------------------------------------
   * Constructors
   *--------------------------------------------------------------------------------
   */

  /**
   * Creates a new <CODE>HttpErrorMessage</CODE> instance.
   *
   * @param code The {@link com.silverwrist.dynamo.HttpStatusCode HttpStatusCode} to be sent by this object.
   */
  public HttpErrorMessage(HttpStatusCode code)
  {
    m_code = code;
    m_message = null;

  } // end constructor

  /**
   * Creates a new <CODE>HttpErrorMessage</CODE> instance.
   *
   * @param code The {@link com.silverwrist.dynamo.HttpStatusCode HttpStatusCode} to be sent by this object.
   * @param message The message to be sent by this object.
   */
  public HttpErrorMessage(HttpStatusCode code, String message)
  {
    m_code = code;
    m_message = message;

  } // end constructor

  /**
   * Creates a new <CODE>HttpErrorMessage</CODE> instance.
   *
   * @param code The {@link com.silverwrist.dynamo.HttpStatusCode HttpStatusCode} to be sent by this object.
   * @param exception The exception object containing the message to be sent by this object.
   */
  public HttpErrorMessage(HttpStatusCode code, Throwable exception)
  {
    m_code = code;
    m_message = exception.getMessage();

  } // end constructor

  /*--------------------------------------------------------------------------------
   * Overrides from class Object
   *--------------------------------------------------------------------------------
   */

  /**
   * Returns a string representation of the object. In general, the <CODE>toString</CODE> method returns a string
   * that "textually represents" this object. 
   *
   * @return A string representation of the object.
   */
  public String toString()
  {
    StringBuffer buf = new StringBuffer("HttpErrorMessage: ");
    buf.append(m_code.getName()).append(" (").append(m_code.getValue()).append(")");
    if (m_message!=null)
      buf.append(" - ").append(m_message);
    return buf.toString();

  } // end toString

  /*--------------------------------------------------------------------------------
   * Implementations from interface SelfRenderable
   *--------------------------------------------------------------------------------
   */

  /**
   * Renders this object to the output.
   *
   * @param control A rendering control object that provides the means to render this object to the output, as
   *                well as providing "output services" (through the
   *                {@link com.silverwrist.dynamo.iface.ServiceProvider ServiceProvider} interface) for the
   *                rendering process to use.
   * @exception java.io.IOException If an I/O error occurs while writing the output.
   * @exception com.silverwrist.dynamo.except.RenderingException If the rendering process fails.
   */
  public void render(SelfRenderControl control) throws IOException, RenderingException
  {
    if (m_message==null)
      control.error(m_code);
    else
      control.error(m_code,m_message);

  } // end render

  /*--------------------------------------------------------------------------------
   * External operations
   *--------------------------------------------------------------------------------
   */

  /**
   * Returns the {@link com.silverwrist.dynamo.HttpStatusCode HttpStatusCode} to be sent by this object.
   *
   * @return The <CODE>HttpStatusCode</CODE> to be sent by this objec.t
   */
  public HttpStatusCode getCode()
  {
    return m_code;

  } // end getCode

  /**
   * Returns the error message to be sent by this object.
   *
   * @return The error message to be sent by this object.
   */
  public String getMessage()
  {
    return m_message;

  } // end getMessage

} // end class HttpErrorMessage
