/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2002-03 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.dynamo.util;

import java.io.IOException;
import com.silverwrist.dynamo.except.RenderingException;
import com.silverwrist.dynamo.iface.*;

/**
 * A Dynamo output object which, when rendered, causes the user's browser to redirect to a specified URL.
 * It sends an HTTP response code of 302 (Redirect) to the client.
 *
 * @author Eric J. Bowersox &lt;erbo@silcom.com&gt;
 * @version X
 */
public class Redirect implements SelfRenderable
{
  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private String m_type;       // redirection type
  private String m_url;        // redirection URL

  /*--------------------------------------------------------------------------------
   * Constructor
   *--------------------------------------------------------------------------------
   */

  /**
   * Constructs a new redirection object.
   *
   * @param type The type of URL this is; used to determine how to rewrite the URL.
   * @param url The URL to redirect the browser to.
   */
  public Redirect(String type, String url)
  {
    m_type = type;
    m_url = url;

  } // end constructor

  /*--------------------------------------------------------------------------------
   * Overrides from class Object
   *--------------------------------------------------------------------------------
   */

  /**
   * Returns a string representation of the object. In general, the <CODE>toString</CODE> method returns a string
   * that "textually represents" this object. 
   *
   * @return A string representation of the object.
   */
  public String toString()
  {
    StringBuffer buf = new StringBuffer("Redirect: {");
    buf.append(m_type).append("} => ").append(m_url);
    return buf.toString();

  } // end toString

  /*--------------------------------------------------------------------------------
   * Implementations from interface SelfRenderable
   *--------------------------------------------------------------------------------
   */

  /**
   * Renders this object to the output.
   *
   * @param control A rendering control object that provides the means to render this object to the output, as
   *                well as providing "output services" (through the
   *                {@link com.silverwrist.dynamo.iface.ServiceProvider ServiceProvider} interface) for the
   *                rendering process to use.
   * @exception java.io.IOException If an I/O error occurs while writing the output.
   * @exception com.silverwrist.dynamo.except.RenderingException If the rendering process fails.
   */
  public void render(SelfRenderControl control) throws IOException, RenderingException
  {
    URLRewriter rewriter = (URLRewriter)(control.queryService(URLRewriter.class));
    control.redirect(rewriter.rewriteRedirectURL(m_type,m_url));

  } // end render

  /*--------------------------------------------------------------------------------
   * External operations
   *--------------------------------------------------------------------------------
   */

  /**
   * Returns the URL type for this redirection.
   *
   * @return The URL type for this redirection.
   */
  public String getType()
  {
    return m_type;

  } // end getType

  /**
   * Returns the URL that this object will redirect to.
   *
   * @return The URL that this object will redirect to.
   */
  public String getURL()
  {
    return m_url;

  } // end getURL

} // end class Redirect
